package main

import (
	"crypto/hmac"
	"crypto/sha256"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"time"
)

func _sign(key []byte, msg string) []byte {
	h := hmac.New(sha256.New, key)
	h.Write([]byte(msg))
	return h.Sum(nil)
}

func get_signature_key(key string, dateStamp string, regionName string, serviceName string) []byte {
	kDate := _sign([]byte("AWS4"+key), dateStamp)
	kRegion := _sign(kDate, regionName)
	kService := _sign(kRegion, serviceName)
	kSigning := _sign(kService, "aws4_request")
	return kSigning
}

func api_request(action, version, access_key, secret_key, host, method, service string,
	req_params map[string]string, post_data map[string]interface{}) (int, string) {

	// ************* REQUEST VALUES *************
	region := "cn-beijing-6"
	endpoint := "http://" + host
	request_parameters := "Action=" + action + "&Version=" + version
	if req_params != nil {
		params := url.Values{}
		for key := range req_params {
			params.Add(key, req_params[key])
		}
		request_parameters += "&" + params.Encode()
	}
	if access_key == "" || secret_key == "" {
		fmt.Println("No access key is available.")
		return 400, ""
	}
	t := time.Now().UTC()
	amzdate := t.Format("20060102T150405Z")
	datestamp := t.Format("20060102")

	// ************* TASK 1: CREATE A CANONICAL REQUEST *************
	// Step 1 is to define the verb (GET, POST, etc.)--already done.

	// Step 2: Create canonical URI--the part of the URI from domain to query
	// string (use '/' if no path)
	canonical_uri := "/"

	// Step 3: Create the canonical query string. In this example (a GET request),
	// request parameters are in the query string. Query string values must
	// be URL-encoded (space=%20). The parameters must be sorted by name.
	// For this example, the query string is pre-formatted in the
	// request_parameters variable.
	canonical_querystring := request_parameters

	// Step 4: Create the canonical headers and signed headers. Header names
	// must be trimmed and lowercase, and sorted in code point order from
	// low to high. Note that there is a trailing \n.
	canonical_headers := "content-type:application/json" + "\n" + "host:" + host + "\n" + "x-amz-date:" + amzdate + "\n"

	// Step 5: Create the list of signed headers. This lists the headers
	// in the canonical_headers list, delimited with ";" and in alpha order.
	// Note: The request can include any headers; canonical_headers and
	// signed_headers lists those that you want to be included in the
	// hash of the request. "Host" and "x-amz-date" are always required.
	signed_headers := "content-type;host;x-amz-date"

	// Step 6: Create payload hash (hash of the request body content). For GET
	// requests, the payload is an empty string ("").
	h := sha256.New()
	var json_data string
	if post_data == nil {
		h.Write([]byte(""))
	} else {
		data, _ := json.Marshal(post_data)
		json_data = string(data)
		//fmt.Println(string(json_data))
		//data := "{\"business\": [\"porn\"], \"image_urls\": [\"https://ks3-cn-beijing.ksyun.com/imgdb/chenshuibian.jpeg\"]}"
		//fmt.Println(data)
		h.Write([]byte(json_data))
	}
	payload_hash := fmt.Sprintf("%x", h.Sum(nil))
	//fmt.Printf("%x\n", payload_hash)

	// Step 7: Combine elements to create canonical request
	canonical_request := method + "\n" + canonical_uri + "\n" + canonical_querystring + "\n" + canonical_headers + "\n" + signed_headers + "\n" + payload_hash

	// ************* TASK 2: CREATE THE STRING TO SIGN*************
	// Match the algorithm to the hashing algorithm you use, either SHA-1 or
	// SHA-256 (recommended)
	algorithm := "AWS4-HMAC-SHA256"
	credential_scope := datestamp + "/" + region + "/" + service + "/" + "aws4_request"
	ha := sha256.New()
	ha.Write([]byte(canonical_request))
	hash := fmt.Sprintf("%x", ha.Sum(nil))
	string_to_sign := algorithm + "\n" + amzdate + "\n" + credential_scope + "\n" + hash

	// ************* TASK 3: CALCULATE THE SIGNATURE *************
	// Create the signing key using the function defined above.
	signing_key := get_signature_key(secret_key, datestamp, region, service)

	// Sign the string_to_sign using the signing_key
	s := hmac.New(sha256.New, signing_key)
	s.Write([]byte(string_to_sign))
	signature := fmt.Sprintf("%x", s.Sum(nil))

	// ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST ***********
	// The signing information can be either in a query string value or in
	// a header named Authorization. This code shows how to use a header.
	// Create authorization header and add to request headers
	authorization_header := algorithm + " " + "Credential=" + access_key + "/" + credential_scope + ", " + "SignedHeaders=" + signed_headers + ", " + "Signature=" + signature

	// ************* SEND THE REQUEST *************
	request_url := endpoint + "?" + canonical_querystring
	client := &http.Client{}
	req, _ := http.NewRequest(method, request_url, strings.NewReader(json_data))
	req.Header.Set("Content-Type", "application/json")
	req.Header.Set("X-Amz-Date", amzdate)
	req.Header.Set("Authorization", authorization_header)
	resp, _ := client.Do(req)
	defer resp.Body.Close()
	body, _ := ioutil.ReadAll(resp.Body)
	code := resp.StatusCode

	return code, string(body)
}

func main() {
	post_data := map[string]interface{}{
		"guard_id":  "1547778774476511751",
		"image_url": "https://ks3-cn-beijing.ksyun.com/datasets/sdk/chenshuibian.png",
	}

	ak := "ak from www.ksyun.com"
	sk := "sk from www.ksyun.com"

	code, text := api_request(
		"ClassifyImageGuard",
		"2019-01-18",
		ak,
		sk,
		"kir.api.ksyun.com",
		"POST",
		"kir",
		nil,
		post_data,
	)
	fmt.Println(code)
	fmt.Println(text)
}
