# -*- coding: utf-8 -*-
# Summary: OpenAPI
# Author: Hongchen Dou
# Time-stamp: Mon Dec  4 20:06:38 2017
import datetime
import hashlib
import hmac
import json
import sys
import requests

if sys.version_info < (3, 0):
    import urllib
else:
    from urllib import parse


def _sign(key, msg):
    return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()


def get_signature_key(key, dateStamp, regionName, serviceName):
    kDate = _sign(('AWS4' + key).encode('utf-8'), dateStamp)
    kRegion = _sign(kDate, regionName)
    kService = _sign(kRegion, serviceName)
    kSigning = _sign(kService, 'aws4_request')
    return kSigning


def api_request(action, version, access_key, secret_key, host='kir.api.ksyun.com', method='POST', service='kir', req_params=None,
                post_data=None):
    """
    openapi 请求
    :param host: 主机名
    :param method: 'GET'/ 'POST'
    :param service: 取值'kir'
    :param action: ClassifyImage
    :param version: API版本号
    :param access_key:
    :param secret_key:
    :param req_params: 请求参数. eg: {"image_url":"https://ks3-cn-beijing.ksyun.com/datasets/sdk/chenshuibian.png"}
    :param post_data: json请求体，参考api文档
    """

    # ************* REQUEST VALUES *************
    region = 'cn-beijing-6'
    endpoint = 'http://{}'.format(host)
    if req_params:
        if sys.version_info < (3, 0):
            request_parameters = 'Action={}&Version={}&'.format(
                action, version) + urllib.urlencode(req_params)
        else:
            request_parameters = 'Action={}&Version={}&'.format(
                action, version) + parse.urlencode(req_params)
    else:
        request_parameters = 'Action={}&Version={}'.format(action, version)

    if access_key is None or secret_key is None:
        print('No access key is available.')
        return (400, None)

    # Create a date for headers and the credential string
    t = datetime.datetime.utcnow()
    amzdate = t.strftime('%Y%m%dT%H%M%SZ')
    datestamp = t.strftime('%Y%m%d')  # Date w/o time, used in credential scope
    #amzdate = '20171129T100303Z'
    #datestamp = '20171129'

    # ************* TASK 1: CREATE A CANONICAL REQUEST *************

    # Step 1 is to define the verb (GET, POST, etc.)--already done.

    # Step 2: Create canonical URI--the part of the URI from domain to query
    # string (use '/' if no path)
    canonical_uri = '/'

    # Step 3: Create the canonical query string. In this example (a GET request),
    # request parameters are in the query string. Query string values must
    # be URL-encoded (space=%20). The parameters must be sorted by name.
    # For this example, the query string is pre-formatted in the request_parameters variable.
    canonical_querystring = request_parameters

    # Step 4: Create the canonical headers and signed headers. Header names
    # must be trimmed and lowercase, and sorted in code point order from
    # low to high. Note that there is a trailing \n.
    canonical_headers = 'content-type:application/json' + '\n' + \
        'host:' + host + '\n' + 'x-amz-date:' + amzdate + '\n'

    # Step 5: Create the list of signed headers. This lists the headers
    # in the canonical_headers list, delimited with ";" and in alpha order.
    # Note: The request can include any headers; canonical_headers and
    # signed_headers lists those that you want to be included in the
    # hash of the request. "Host" and "x-amz-date" are always required.
    signed_headers = 'content-type;host;x-amz-date'

    # Step 6: Create payload hash (hash of the request body content). For GET
    # requests, the payload is an empty string ("").
    if post_data is None:
        payload_hash = hashlib.sha256(''.encode('utf-8')).hexdigest()
    else:
        payload_hash = hashlib.sha256(post_data.encode('utf-8')).hexdigest()

    # Step 7: Combine elements to create canonical request
    canonical_request = method + '\n' + canonical_uri + '\n' + canonical_querystring + \
        '\n' + canonical_headers + '\n' + signed_headers + '\n' + payload_hash

    # ************* TASK 2: CREATE THE STRING TO SIGN*************
    # Match the algorithm to the hashing algorithm you use, either SHA-1 or
    # SHA-256 (recommended)
    algorithm = 'AWS4-HMAC-SHA256'
    credential_scope = datestamp + '/' + region + \
        '/' + service + '/' + 'aws4_request'
    string_to_sign = algorithm + '\n' + amzdate + '\n' + credential_scope + \
        '\n' + hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()

    # ************* TASK 3: CALCULATE THE SIGNATURE *************
    # Create the signing key using the function defined above.
    signing_key = get_signature_key(secret_key, datestamp, region, service)

    # Sign the string_to_sign using the signing_key
    signature = hmac.new(signing_key, (string_to_sign).encode(
        'utf-8'), hashlib.sha256).hexdigest()

    # ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST *************
    # The signing information can be either in a query string value or in
    # a header named Authorization. This code shows how to use a header.
    # Create authorization header and add to request headers
    authorization_header = algorithm + ' ' + 'Credential=' + access_key + '/' + \
        credential_scope + ', ' + 'SignedHeaders=' + \
        signed_headers + ', ' + 'Signature=' + signature

    # The request can include any headers, but MUST include "host", "x-amz-date",
    # and (for this scenario) "Authorization". "host" and "x-amz-date" must
    # be included in the canonical_headers and signed_headers, as noted
    # earlier. Order here is not significant.
    # Python note: The 'host' header is added automatically by the Python 'requests' library.
    headers = {'Content-Type': 'application/json',
               'X-Amz-Date': amzdate, 'Authorization': authorization_header}

    # ************* SEND THE REQUEST *************
    request_url = endpoint + '?' + canonical_querystring

    # print '\nBEGIN REQUEST++++++++++++++++++++++++++++++++++++'
    # print 'Request URL = ' + request_url
    r = requests.request(method, request_url, headers=headers, data=post_data)
    return (r.status_code, r.text)


if __name__ == "__main__":
    post_data = {
        "guard_id": "1547778774476511751",
        "image_url": "https://ks3-cn-beijing.ksyun.com/datasets/sdk/chenshuibian.png"
    }

    ak = "ak from www.ksyun.com"
    sk = "sk from www.ksyun.com"
    code, text = api_request(
        host='kir.api.ksyun.com',
        method='POST',
        service='kir',
        action='ClassifyImageGuard',
        version='2019-01-18',
        access_key=ak,
        secret_key=sk,
        post_data=json.dumps(post_data)
    )
    print(text)
